#define FakeAccelerometer

using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

using System.IO;
using System.Net;

namespace PhoneStarlight
{

    public class AccelerometerState
    {
        public Vector3 Acceleration;

        public AccelerometerState()
        {
        }
    }

#if FakeAccelerometer

    public class Accelerometer
    {
        static string url = "http://10.22.1.190:8080/Acellerometer";

        static bool pending = false;

        public static bool Update()
        {
            if (pending) return false;
            pending = true;
            Uri uri = new Uri(url);

            WebClient client = new WebClient();
            client.DownloadStringCompleted += new DownloadStringCompletedEventHandler(client_DownloadStringCompleted);
            client.DownloadStringAsync(uri);
            return true;
        }

        static void client_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                string[] numbers = e.Result.Split(new char[] { ',' });
                state.Acceleration.X = float.Parse(numbers[0]);
                state.Acceleration.Y = float.Parse(numbers[1]);
                state.Acceleration.Z = float.Parse(numbers[2]);
            }
            pending = false;
        }

        static AccelerometerState state = new AccelerometerState();

        public static AccelerometerState GetState()
        {
            return state;
        }
    }

#else

    public class Accelerometer
    {
        static Accelerometer ()
        {
            Microsoft.Devices.Sensors.AccelerometerSensor.Default.ReadingChanged += 
                new EventHandler<Microsoft.Devices.Sensors.AccelerometerReadingAsyncEventArgs>(Default_ReadingChanged);
        }

        static AccelerometerState state = new AccelerometerState();

        static void Default_ReadingChanged(object sender, Microsoft.Devices.Sensors.AccelerometerReadingAsyncEventArgs e)
        {
            state.Acceleration.X = (float)e.Value.Value.X;
            state.Acceleration.Y = (float)e.Value.Value.Y;
            state.Acceleration.Z = (float)e.Value.Value.Z;
        }

        public static AccelerometerState GetState()
        {
            KeyboardState keys = Keyboard.GetState();

            if (keys.GetPressedKeys().Length > 0)
            {
                Keys[] keysPressed = keys.GetPressedKeys();

                foreach ( Keys k in keysPressed ) 
                {
                    Console.Write(k.ToString() + " ");
                }
                Console.WriteLine();

                if (keys.IsKeyDown(Keys.LeftAlt))
                {
                    if (keys.IsKeyDown(Keys.Right))
                    {
                        state.Acceleration.X += 0.01f;
                    }
                    if (keys.IsKeyDown(Keys.Left))
                    {
                        state.Acceleration.X -= 0.01f;
                    }
                    if (keys.IsKeyDown(Keys.Up))
                    {
                        state.Acceleration.Y += 0.01f;
                    }
                    if (keys.IsKeyDown(Keys.Down))
                    {
                        state.Acceleration.Y -= 0.01f;
                    }
                }
            }
            return state;
        }
    }

#endif

    public interface ISpriteBasedGame
    {
        void UpdateLives(int update);
        void UpdateScore(int update);
        int GetScore();
        GamePadState GetGamePad();
        TouchCollection GetTouchPad();
        AccelerometerState GetAccellerometer();
        void StartGame();
        void EndGame();
    }

    public class BaseSprite
    {
        protected Texture2D spriteTexture;
        protected Rectangle spriteRectangle;

        /// <summary>
        /// Loads the texture to be used in the sprite.
        /// </summary>
        /// <param name="spriteTexture">The texture to be used.</param>
        public void LoadTexture(Texture2D inSpriteTexture)
        {
            spriteTexture = inSpriteTexture;
        }

        /// <summary>
        /// Loads the rectangle to be used as the destination for 
        /// draw operations for this background sprite.
        /// </summary>
        /// <param name="inSpriteRectangle">The rectangle to be used.</param>
        public void SetRectangle(Rectangle inSpriteRectangle)
        {
            spriteRectangle = inSpriteRectangle;
        }

        /// <summary>
        /// Called when the game starts running
        /// </summary>
        public virtual void StartGame()
        {
        }

        /// <summary>
        /// Called when the game stops running
        /// </summary>
        public virtual void EndGame()
        {
        }

        /// <summary>
        /// Draws the texture using the rectangle that dimensions it.
        /// </summary>
        /// <param name="spriteBatch">The SpriteBatch to be used
        /// for the drawing operation.</param>
        public virtual void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(spriteTexture, spriteRectangle, Color.White);
        }

        /// <summary>
        /// Update behaviour for the title. The base sprite does not have
        /// any update behaviour, but other sprites might.
        /// </summary>
        /// <param name="game">Game to be controlled.</param>
        public virtual void Update(ISpriteBasedGame game)
        {
        }

        public BaseSprite(Texture2D inSpriteTexture, Rectangle inRectangle)
        {
            LoadTexture(inSpriteTexture);
            SetRectangle(inRectangle);
        }

    }

    public class TitleSprite : BaseSprite
    {
        /// <summary>
        /// Update behaviour for the title. The A button is tested and the game
        /// is started if it is pressed.
        /// </summary>
        /// <param name="game">Game to be controlled.</param>
        public override void Update(ISpriteBasedGame game)
        {
            TouchCollection touchList = game.GetTouchPad();

            if (touchList.Count > 0)
            {
                foreach (TouchLocation touch in touchList)
                {
                    if (touch.State == TouchLocationState.Released)
                    {
                        game.StartGame();
                        return;
                    }
                }
            }
            if (game.GetGamePad().Buttons.A == ButtonState.Pressed)
            {
                game.StartGame();
            }
        }

        public TitleSprite(Texture2D inSpriteTexture, Rectangle inRectangle)
            : base(inSpriteTexture, inRectangle)
        {
        }
    }

    public class MovingSprite : BaseSprite
    {
        protected float x;
        protected float y;
        protected float xSpeed;
        protected float ySpeed;

        protected float initialX;
        protected float initialY;

        protected float minDisplayX;
        protected float maxDisplayX;

        protected float minDisplayY;
        protected float maxDisplayY;

        public float XPos
        {
            get
            {
                return x;
            }
        }

        public float YPos
        {
            get
            {
                return y;
            }
        }

        /// <summary>
        /// Check for collisions between the sprite and other objects.
        /// </summary>
        /// <param name="target">Rectangle giving the position of the other object.</param>
        /// <returns>true if the target has collided with this object</returns>
        public virtual bool CheckCollision(Rectangle target)
        {
            return spriteRectangle.Intersects(target);
        }

        /// <summary>
        /// Creates a moving sprite
        /// </summary>
        /// <param name="spriteTexture">Texture to use for the sprite</param>
        /// <param name="widthFactor">Size of the game object, as a fraction of the width
        /// of the screen.</param>
        /// <param name="ticksToCrossScreen">Speed of movement, in number of 60th of a second ticks to 
        /// cross the entire screen.</param>
        /// <param name="minDisplayX">minimum X value</param>
        /// <param name="maxDisplayX">maximum X value</param>
        /// <param name="minDisplayY">minimum Y value</param>
        /// <param name="maxDisplayY">maximum Y value</param>
        /// <param name="initialX">start X position for the sprite</param>
        /// <param name="initialY">start Y position for the sprite</param>
        public MovingSprite(
            Texture2D inSpriteTexture,
            float widthFactor,
            float ticksToCrossScreen,
            float inMinDisplayX,
            float inMaxDisplayX,
            float inMinDisplayY,
            float inMaxDisplayY,
            float inInitialX,
            float inInitialY)
            : base(inSpriteTexture, Rectangle.Empty)
        {
            minDisplayX = inMinDisplayX;
            minDisplayY = inMinDisplayY;
            maxDisplayX = inMaxDisplayX;
            maxDisplayY = inMaxDisplayY;
            initialX = inInitialX;
            initialY = inInitialY;

            float displayWidth = maxDisplayX - minDisplayX;

            spriteRectangle.Width = (int)((displayWidth * widthFactor) + 0.5f);
            float aspectRatio =
                (float)spriteTexture.Width / spriteTexture.Height;
            spriteRectangle.Height =
                (int)((spriteRectangle.Width / aspectRatio) + 0.5f);
            x = initialX;
            y = initialY;
            xSpeed = displayWidth / ticksToCrossScreen;
            ySpeed = xSpeed;
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            base.Draw(spriteBatch);
        }

        public override void Update(ISpriteBasedGame game)
        {
            spriteRectangle.X = (int)(x + 0.5f);
            spriteRectangle.Y = (int)(y + 0.5f);
            base.Update(game);
        }

        public override void StartGame()
        {
            x = initialX;
            y = initialY;
            spriteRectangle.X = (int)x;
            spriteRectangle.Y = (int)y;
            base.StartGame();
        }
    }

    public class BackgroundStar : MovingSprite
    {
        /// <summary>
        /// Draws a white background star that travels down the screen.
        /// </summary>
        /// <param name="spriteTexture">Texture for the star</param>
        /// <param name="widthFactor">Fraction of the width of the screen the star will fill</param>
        /// <param name="ticksToCrossScreen">Number of ticks to travel down the screen</param>
        /// <param name="minDisplayX">Minimum X value of display area</param>
        /// <param name="maxDisplayX">Maximum X value of display area</param>
        /// <param name="minDisplayY">Minimum Y value of display area</param>
        /// <param name="maxDisplayY">Maximum Y value of display area</param>
        /// <param name="initialX">Initial X position of star</param>
        /// <param name="initialY">Initial Y position of star</param>
        public BackgroundStar(
            Texture2D spriteTexture,
            float widthFactor, float ticksToCrossScreen,
            float minDisplayX, float maxDisplayX,
            float minDisplayY, float maxDisplayY,
            float initialX, float initialY)
            : base(
                spriteTexture, widthFactor,
                ticksToCrossScreen, minDisplayX, maxDisplayX,
                minDisplayY, maxDisplayY, initialX, initialY)
        {
        }

        /// <summary>
        /// Moves the sprite down the screen and replaces it at the top if it 
        /// goes off the bottom
        /// </summary>
        /// <param name="game">Game the sprite is part of</param>
        public override void Update(ISpriteBasedGame game)
        {
            y += ySpeed;

            if (y > maxDisplayY)
            {
                y = minDisplayY - spriteRectangle.Height;
            }
            base.Update(game);
        }
    }

    public class AnimatedSprite : MovingSprite
    {
        /// <summary>
        /// Source rectangle to take out of the sprite texture
        /// </summary>
        private Rectangle sourceRect;

        /// <summary>
        /// Width of one frame in the animation
        /// </summary>
        private int frameWidth;
        /// <summary>
        /// Height of one frame in the animation
        /// </summary>
        private int frameHeight;

        /// <summary>
        /// Counts the updates
        /// When it reaches 5 it is time to update the frames
        /// </summary>
        protected int updateTickCounter = 0;

        /// <summary>
        /// Creates a new animated sprite.
        /// </summary>
        /// <param name="inSpriteTexture">Source texture containing multiple sprite images</param>
        /// <param name="widthFactor">Fraction of the width of the screen the star will fill</param>
        /// <param name="ticksToCrossScreen">Number of ticks to travel down the screen</param>
        /// <param name="minDisplayX">Minimum X value of display area</param>
        /// <param name="maxDisplayX">Maximum X value of display area</param>
        /// <param name="minDisplayY">Minimum Y value of display area</param>
        /// <param name="maxDisplayY">Maximum Y value of display area</param>
        /// <param name="initialX">Initial X position of sprite</param>
        /// <param name="initialY">Initial Y position of sprite</param>
        /// <param name="inFrameWidth">width of each frame of the animation</param>
        /// <param name="inFrameHeight">height of each frame of the animation</param>
        public AnimatedSprite(
            Texture2D inSpriteTexture,
            float widthFactor, float ticksToCrossScreen,
            float minDisplayX, float maxDisplayX,
            float minDisplayY, float maxDisplayY,
            float initialX, float initialY,
            int inFrameWidth, int inFrameHeight)
            : base(
                inSpriteTexture, widthFactor,
                ticksToCrossScreen, minDisplayX, maxDisplayX,
            minDisplayY, maxDisplayY, initialX, initialY)
        {
            frameWidth = inFrameWidth;
            frameHeight = inFrameHeight;
            sourceRect = new Rectangle(0, 0, inFrameWidth, inFrameHeight);

            // We must use the aspect ratio of a frame in the animation
            // to scale the sprite, not the size of all the frames
            float aspectRatio = (float)frameWidth / frameHeight;
            spriteRectangle.Height = (int)((spriteRectangle.Width / aspectRatio) + 0.5f);
        }

        public override void Update(ISpriteBasedGame game)
        {
            updateTickCounter++;

            if (updateTickCounter == 5)
            {
                updateTickCounter = 0;
                if (sourceRect.X + frameWidth >= spriteTexture.Width)
                {
                    // reset the animation to the start frame
                    sourceRect.X = 0;
                }
                else
                {
                    // Move on to the next frame
                    sourceRect.X += frameWidth;
                }
            }
            base.Update(game);
        }

        /// <summary>
        /// Number of the row currently selected for the display
        /// </summary>
        private int rowNumber;

        /// <summary>
        /// Gets the currently active row on the display
        /// </summary>
        /// <returns>The currently displayed row on the texture</returns>
        public int getRowNumber()
        {
            return rowNumber;
        }

        /// <summary>
        /// Selects a particular display row on the texture.
        /// </summary>
        /// <param name="newRowNuumber">Number of the new row. The row at 
        /// the top is number 0</param>
        /// <returns>true if the row was set correctly, 
        /// false if the source texture does not contain that row</returns>
        public bool SetRow(int newRowNuumber)
        {
            int rowY = newRowNuumber * frameHeight;

            if (rowY + frameHeight > spriteTexture.Height)
            {
                // This row does not exist
                return false;
            }

            sourceRect.Y = rowY;
            rowNumber = newRowNuumber;
            return true;
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(spriteTexture, spriteRectangle, sourceRect, Color.White);
        }
    }

    public class ExplodingSprite : AnimatedSprite
    {
        private Texture2D explodeTexture;
        private Rectangle explodeSourceRect;
        private int explodeFrameWidth;
        private int explodeTickCounter;
        private bool exploding;
        private SoundEffect explodeSound;

        /// <summary>
        /// Creates a new animated sprite.
        /// </summary>
        /// <param name="inSpriteTexture">Source texture containing multiple sprite images</param>
        /// <param name="widthFactor">Fraction of the width of the screen the star will fill</param>
        /// <param name="ticksToCrossScreen">Number of ticks to travel down the screen</param>
        /// <param name="minDisplayX">Minimum X value of display area</param>
        /// <param name="maxDisplayX">Maximum X value of display area</param>
        /// <param name="minDisplayY">Minimum Y value of display area</param>
        /// <param name="maxDisplayY">Maximum Y value of display area</param>
        /// <param name="initialX">Initial X position of sprite</param>
        /// <param name="initialY">Initial Y position of sprite</param>
        /// <param name="inFrameWidth">width of each frame of the animation</param>
        /// <param name="inFrameHeight">height of each frame of the animation</param>
        /// <param name="inExplodeTexture">texture for the explosion</param>
        /// <param name="inExplodeFrameWidth">width of frame in explosion animation</param>
        /// <param name="inExplodeSound">sound to play when the sprite explodes</param>
        public ExplodingSprite(
            Texture2D inSpriteTexture,
            float widthFactor, float ticksToCrossScreen,
            float minDisplayX, float maxDisplayX,
            float minDisplayY, float maxDisplayY,
            float initialX, float initialY,
            int inFrameWidth, int inFrameHeight,
            Texture2D inExplodeTexture,
            int inExplodeFrameWidth,
            SoundEffect inExplodeSound)
            : base(
                inSpriteTexture, widthFactor,
                ticksToCrossScreen, minDisplayX, maxDisplayX,
                minDisplayY, maxDisplayY,
                initialX, initialY,
                inFrameWidth, inFrameHeight)
        {
            explodeTexture = inExplodeTexture;
            explodeFrameWidth = inExplodeFrameWidth;
            explodeSourceRect = new Rectangle(
                0, 0,
                explodeFrameWidth,
                explodeTexture.Height);
            explodeSound = inExplodeSound;
        }

        public override void Update(ISpriteBasedGame game)
        {
            if (exploding)
            {
                explodeTickCounter++;
                if (explodeTickCounter == 5)
                {
                    explodeTickCounter = 0;
                    if (explodeSourceRect.X + explodeFrameWidth >= explodeTexture.Width)
                    {
                        // reached the end of the sequence
                        // not exploding any more
                        exploding = false;
                    }
                    else
                    {
                        // Move on to the next frame
                        explodeSourceRect.X += explodeFrameWidth;
                    }
                }
            }
            base.Update(game);
        }

        public override void StartGame()
        {
            exploding = false;
            base.StartGame();
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            base.Draw(spriteBatch);
            if (exploding)
            {
                spriteBatch.Draw(explodeTexture, spriteRectangle,
                                 explodeSourceRect, Color.White);
            }
        }

        public bool IsExploding
        {
            get
            {
                return exploding;
            }
        }

        public void Explode()
        {
            if (exploding)
            {
                return;
            }
            explodeSound.Play();
            explodeSourceRect.X = 0;
            explodeTickCounter = 0;
            exploding = true;
        }
    }

    public class RocketSprite : ExplodingSprite
    {
        public RocketSprite(
            Texture2D inSpriteTexture,
            float widthFactor, float ticksToCrossScreen,
            float minDisplayX, float maxDisplayX,
            float minDisplayY, float maxDisplayY,
            float initialX, float initialY,
            int inFrameWidth, int inFrameHeight,
            Texture2D inExplodeTexture,
            int inExplodeFrameWidth,
            SoundEffect inExplodeSound)
            : base(
                inSpriteTexture, widthFactor,
                ticksToCrossScreen, minDisplayX, maxDisplayX,
                minDisplayY, maxDisplayY,
                initialX, initialY,
                inFrameWidth, inFrameHeight,
                inExplodeTexture,
                inExplodeFrameWidth,
                inExplodeSound)
        {
        }

        public override void Update(ISpriteBasedGame game)
        {
            x = x + (xSpeed * game.GetGamePad().ThumbSticks.Left.X);
            y = y - (ySpeed * game.GetGamePad().ThumbSticks.Left.Y);

            AccelerometerState accell = game.GetAccellerometer();

            x += 5.0f * accell.Acceleration.X;
            y += -5.0f * accell.Acceleration.Y;

            // Make sure it is still on the screen:

            if (x < minDisplayX)
            {
                x = minDisplayX;
            }

            if (x + spriteRectangle.Width >= maxDisplayX)
            {
                x = maxDisplayX - spriteRectangle.Width;
            }

            if (y < minDisplayY)
            {
                y = minDisplayY;
            }

            if (y + spriteRectangle.Height >= maxDisplayY)
            {
                y = maxDisplayY - spriteRectangle.Height;
            }

            base.Update(game);
        }

        public override void StartGame()
        {
            SetRow(0);
            base.StartGame();
        }

        /// <summary>
        /// Gets the X position for a missile launch
        /// </summary>
        /// <returns>The X position for a missile launch from this rocket.</returns>
        public float GetMissileLaunchX()
        {
            return XPos + (spriteRectangle.Width / 2);
        }

        /// <summary>
        /// Called when we have been hit
        /// </summary>
        public void Hit()
        {
            Explode();
            SetRow(getRowNumber() + 1);
        }
    }

    public class AlienSprite : ExplodingSprite
    {
        private RocketSprite rocket;

        private float initialXSpeed;
        private float initialYSpeed;

        public AlienSprite(
            Texture2D inSpriteTexture,
            float widthFactor, float ticksToCrossScreen,
            float minDisplayX, float maxDisplayX,
            float minDisplayY, float maxDisplayY,
            float initialX, float initialY,
            int inFrameWidth, int inFrameHeight,
            Texture2D inExplodeTexture,
            int inExplodeFrameWidth,
            SoundEffect inExplodeSound,
            RocketSprite inRocket)
            : base(
                inSpriteTexture, widthFactor,
                ticksToCrossScreen, minDisplayX, maxDisplayX,
                minDisplayY, maxDisplayY,
                initialX, initialY,
                inFrameWidth, inFrameHeight,
                inExplodeTexture,
                inExplodeFrameWidth,
                inExplodeSound)
        {
            rocket = inRocket;
            initialXSpeed = xSpeed;
            initialYSpeed = ySpeed;
        }

        public enum alienSpriteState
        {
            moving,
            chasing,
            killed
        }

        public alienSpriteState State
        {
            get
            {
                return state;
            }
        }

        private alienSpriteState state;

        public void HitAlien()
        {
            Explode();

            switch (state)
            {
                case alienSpriteState.moving:
                    // Show the damaged version of the sprite
                    SetRow(1);
                    xSpeed = 0;
                    ySpeed = 0;
                    state = alienSpriteState.chasing;
                    break;
                case alienSpriteState.chasing:
                    state = alienSpriteState.killed;
                    break;
                case alienSpriteState.killed:
                    break;
            }
        }

        public override void StartGame()
        {
            SetRow(0);
            state = alienSpriteState.moving;
            x = initialX;
            y = initialY;
            xSpeed = initialXSpeed;
            ySpeed = initialYSpeed;
            base.StartGame();
        }

        float alienXMaxAccell = 0.08f;
        float alienYMaxAccell = 0.08f;

        public override void Update(ISpriteBasedGame game)
        {
            switch (state)
            {
                case alienSpriteState.moving:
                    x += xSpeed;
                    if (x + spriteRectangle.Width >= maxDisplayX)
                    {
                        x = maxDisplayX - spriteRectangle.Width;
                        xSpeed = -Math.Abs(xSpeed);
                    }
                    if (x <= minDisplayX)
                    {
                        x = minDisplayX;
                        xSpeed = Math.Abs(xSpeed);
                    }
                    if (rocket.CheckCollision(spriteRectangle))
                    {
                        game.UpdateLives(-1);
                        rocket.Hit();
                        Explode();
                        state = alienSpriteState.killed;
                    }
                    break;

                case alienSpriteState.chasing:

                    float xTargetDistance = x - rocket.XPos;
                    float yTargetDistance = y - rocket.YPos;
                    float displayWidth = maxDisplayX - minDisplayX;
                    float displayHeight = maxDisplayY - minDisplayY;

                    xSpeed = xSpeed + (alienXMaxAccell * (xTargetDistance / displayWidth));
                    ySpeed = ySpeed + (alienYMaxAccell * (yTargetDistance / displayHeight));

                    x = x - xSpeed;
                    y = y - ySpeed;

                    if (x + spriteRectangle.Width >= maxDisplayX)
                    {
                        x = maxDisplayX - spriteRectangle.Width;
                        xSpeed = -Math.Abs(xSpeed);
                    }
                    if (x <= minDisplayX)
                    {
                        x = minDisplayX;
                        xSpeed = Math.Abs(xSpeed);
                    }

                    if (y + spriteRectangle.Height >= maxDisplayY)
                    {
                        y = maxDisplayY - spriteRectangle.Width;
                        ySpeed = -Math.Abs(ySpeed);
                    }
                    if (y <= minDisplayY)
                    {
                        y = minDisplayY;
                        ySpeed = Math.Abs(ySpeed);
                    }
                    if (rocket.CheckCollision(spriteRectangle))
                    {
                        game.UpdateLives(-1);
                        rocket.Hit();
                        Explode();
                        state = alienSpriteState.killed;
                    }
                    break;
                case alienSpriteState.killed:
                    break;
            }
            base.Update(game);
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            if (IsExploding)
            {
                /// Always draw if we are exploding
                base.Draw(spriteBatch);
            }
            else
            {
                // Only draw if we have not been killed
                if (state != alienSpriteState.killed)
                {
                    base.Draw(spriteBatch);
                }
            }
        }
    }

    public class MissileSprite : AnimatedSprite
    {
        private List<AlienSprite> alienSprites;

        private RocketSprite rocket;

        public MissileSprite(
            Texture2D inSpriteTexture,
            float widthFactor, float ticksToCrossScreen,
            float minDisplayX, float maxDisplayX,
            float minDisplayY, float maxDisplayY,
            float initialX, float initialY,
            int inFrameWidth, int inFrameHeight,
            RocketSprite inRocket,
            List<AlienSprite> inAlienSprites
            )
            : base(
                inSpriteTexture, widthFactor,
                ticksToCrossScreen, minDisplayX, maxDisplayX,
                minDisplayY, maxDisplayY,
                initialX, initialY,
                inFrameWidth, inFrameHeight)
        {
            rocket = inRocket;
            alienSprites = inAlienSprites;
        }

        bool flying;

        public override void Update(ISpriteBasedGame game)
        {
            if (flying)
            {
                // Move up the screen and see if we have hit anything
                y = y - ySpeed;
                foreach (AlienSprite alien in alienSprites)
                {
                    if (alien.State == AlienSprite.alienSpriteState.killed)
                    {
                        continue;
                    }
                    if (alien.CheckCollision(spriteRectangle))
                    {
                        game.UpdateScore(50);
                        alien.HitAlien();
                        flying = false;
                        break;
                    }
                }

                if (y < minDisplayY)
                {
                    flying = false;
                }
            }
            else
            {

                if (game.GetTouchPad().Count > 0)
                {
                    x = rocket.GetMissileLaunchX() - spriteRectangle.Width / 2;
                    y = rocket.YPos;
                    flying = true;
                }
                if (game.GetGamePad().Buttons.B == ButtonState.Pressed)
                {
                    x = rocket.GetMissileLaunchX() - spriteRectangle.Width / 2;
                    y = rocket.YPos;
                    flying = true;
                }
            }

            base.Update(game);
        }

        public override void StartGame()
        {
            flying = false;
            base.StartGame();
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            if (flying)
            {
                base.Draw(spriteBatch);
            }
        }

    }

    public class FrameRateCounter : DrawableGameComponent
    {
        ContentManager content;
        SpriteBatch spriteBatch;
        SpriteFont spriteFont;

        int frameRate = 0;
        int frameCounter = 0;
        TimeSpan elapsedTime = TimeSpan.Zero;


        public FrameRateCounter(Game game)
            : base(game)
        {
            content = game.Content;
        }


        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            spriteFont = content.Load<SpriteFont>("Font");
        }


        protected override void UnloadContent()
        {
            content.Unload();
        }


        public override void Update(GameTime gameTime)
        {
            elapsedTime += gameTime.ElapsedGameTime;

            if (elapsedTime > TimeSpan.FromSeconds(1))
            {
                elapsedTime -= TimeSpan.FromSeconds(1);
                frameRate = frameCounter;
                frameCounter = 0;
            }
        }


        public override void Draw(GameTime gameTime)
        {
            frameCounter++;

            string fps = string.Format("fps: {0}", frameRate);

            spriteBatch.Begin();

            spriteBatch.DrawString(spriteFont, fps, new Vector2(33, 33), Color.Black);
            spriteBatch.DrawString(spriteFont, fps, new Vector2(32, 32), Color.White);

            spriteBatch.End();
        }
    }

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class StartLightGame : Microsoft.Xna.Framework.Game, ISpriteBasedGame
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        private GamePadState gamePad1;

        private TouchCollection touchPad;

        private AccelerometerState accellerometer;

        public TitleSprite Title;

        public RocketSprite Rocket;

        public MissileSprite Missile;

        public List<BaseSprite> TitleSprites = new List<BaseSprite>();
        public List<AlienSprite> AlienSprites = new List<AlienSprite>();
        public List<BaseSprite> GameSprites = new List<BaseSprite>();

        /// <summary>
        /// Random number generator. Seeded from a constant value
        /// so that the random sequence is always the same one.
        /// </summary>
        private Random rand = new Random(1);

        #region Display dimension values

        int displayWidth;
        int displayHeight;
        float overScanPercentage = 5.0f;
        float minDisplayX;
        float maxDisplayX;
        float minDisplayY;
        float maxDisplayY;

        private void setScreenSizes()
        {
            displayWidth = graphics.GraphicsDevice.Viewport.Width;
            displayHeight = graphics.GraphicsDevice.Viewport.Height;
            float xOverscanMargin = getPercentage(overScanPercentage, displayWidth) / 2.0f;
            float yOverscanMargin = getPercentage(overScanPercentage, displayHeight) / 2.0f;

            minDisplayX = xOverscanMargin;
            minDisplayY = yOverscanMargin;

            maxDisplayX = displayWidth - xOverscanMargin;
            maxDisplayY = displayHeight - yOverscanMargin;
        }

        #endregion

        #region Utility methods

        /// <summary>
        /// Calculates percentages
        /// </summary>
        /// <param name="percentage">the percentage to be calculated</param>
        /// <param name="inputValue">the value to be worked on</param>
        /// <returns>the resulting value</returns>
        float getPercentage(float percentage, float inputValue)
        {
            return (inputValue * percentage) / 100;
        }

        #endregion

        #region Text drawing

        SpriteFont font;

        private void loadFont()
        {
            font = Content.Load<SpriteFont>("MessageFont");
        }

        /// <summary>
        /// Draws text on the screen
        /// </summary>
        /// <param name="text">text to write</param>
        /// <param name="textColor">color of text</param>
        /// <param name="x">left hand edge of text</param>
        /// <param name="y">top of text</param>
        void drawText(string text, Color textColor, float x, float y)
        {
            int layer;
            Vector2 textVector = new Vector2(x, y);

            // Draw the shadow
            Color backColor = new Color(0, 0, 0, 10);
            for (layer = 0; layer < 10; layer++)
            {
                spriteBatch.DrawString(font, text, textVector, backColor);
                textVector.X++;
                textVector.Y++;
            }

            // Draw the solid part of the characters
            backColor = new Color(190, 190, 190);
            for (layer = 0; layer < 5; layer++)
            {
                spriteBatch.DrawString(font, text, textVector, backColor);
                textVector.X++;
                textVector.Y++;
            }

            // Draw the top of the characters
            spriteBatch.DrawString(font, text, textVector, textColor);
        }

        #endregion

        #region Score Display

        void drawScore()
        {
            drawText("Score : " + score + " Lives : " + lives, Color.Blue, minDisplayX, maxDisplayY - 50);
        }

        void drawHighScore()
        {
            drawText("Highscore : " + highScore, Color.Blue, minDisplayX, maxDisplayY - 50);
        }

        #endregion

        #region Sound Management

        SoundEffect gameOverSound;
        Song musicSong;

        public void StartMusic()
        {
            // Start the musicSound playing
            MediaPlayer.Play(musicSong);
        }

        public void GameOverSound()
        {
            gameOverSound.Play();
        }

        public void StopMusic()
        {
            MediaPlayer.Stop();
        }

        #endregion

        #region Game and Score Management

        int score;
        int lives;
        int highScore;

        public enum GameState
        {
            titleScreen,
            playingGame
        }

        GameState state = GameState.titleScreen;

        public void StartGame()
        {
            score = 0;
            lives = 3;

            foreach (BaseSprite sprite in GameSprites)
            {
                sprite.StartGame();
            }

            StartMusic();

            state = GameState.playingGame;
        }

        public GamePadState GetGamePad()
        {
            return gamePad1;
        }

        public TouchCollection GetTouchPad()
        {
            return touchPad;
        }

        public AccelerometerState GetAccellerometer()
        {
            return accellerometer;
        }

        public void EndGame()
        {
            if (score > highScore)
            {
                highScore = score;
            }

            foreach (BaseSprite sprite in GameSprites)
            {
                sprite.EndGame();
            }

            StopMusic();

            GameOverSound();

            state = GameState.titleScreen;
        }

        public void UpdateLives(int update)
        {
            lives = lives + update;
            if (lives <= 0)
            {
                EndGame();
            }
        }

        public void UpdateScore(int update)
        {
            score += update;
        }

        public int GetScore()
        {
            return score;
        }

        public int GetLives()
        {
            return lives;
        }

        public GameState GetState()
        {
            return state;
        }

        #endregion




        public StartLightGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromSeconds(1 / 30.0);

            //Components.Add(new FrameRateCounter(this));
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            setScreenSizes();

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Title = new TitleSprite(
                Content.Load<Texture2D>("Images/TitleText"),
                new Rectangle((int)minDisplayX, (int)minDisplayY,
                              (int)(maxDisplayX - minDisplayX),
                              (int)(maxDisplayY - minDisplayY)
                 ));

            for (int i = 0; i < 500; i++)
            {
                BackgroundStar star;

                float initialX = rand.Next(0, displayWidth);
                float initialY = rand.Next(0, displayHeight);
                int speed = 100 + rand.Next(500);

                star = new BackgroundStar(
                    Content.Load<Texture2D>("Images/Star"),
                    2.0f / speed,
                    speed,
                    0,
                    displayWidth,
                    0,
                    displayHeight,
                    initialX,
                    initialY);
                GameSprites.Add(star);
                TitleSprites.Add(star);
            }

            TitleSprites.Add(Title);

            Rocket = new RocketSprite(
                    Content.Load<Texture2D>("Sprites/ship2727"),
                    0.14f,   // ship is a 10th the width of the screen
                    100,    // 100 ticks to cros the screen
                    minDisplayX, maxDisplayX, // range of X values
                    minDisplayY, maxDisplayY, // range of Y values
                    maxDisplayX / 2, maxDisplayY / 2, // start position
                    27, 27,  // width and height of frame
                    Content.Load<Texture2D>("Sprites/explosion2727"), // Explosion texture
                    27, // width of each frame in the explosion animation
                    Content.Load<SoundEffect>("Sounds/Explosion") // explosion soundeffect
                    );

            GameSprites.Add(Rocket);

            float alienLevelY = minDisplayY;
            float levelGap = displayHeight / 10;
            float alienXPosition = minDisplayX;

            for (int level = 0; level < 4; level++)
            {
                AlienSprite alien;

                float rowFraction = 0.9f;
                int numberOfAliens = 10;
                alienXPosition = minDisplayX;

                float moveFraction = 1.0f - rowFraction;
                float alienWidth = (maxDisplayX - minDisplayX) * rowFraction / numberOfAliens;
                float alienMoveDistance = moveFraction * (maxDisplayX - minDisplayX);

                for (int i = 0; i < numberOfAliens; i++)
                {
                    alien = new AlienSprite(
                        Content.Load<Texture2D>("Sprites/alien3232"),
                        rowFraction / numberOfAliens,
                        100,    // 100 ticks to cros the screen
                        minDisplayX, maxDisplayX, // range of X values
                        minDisplayY, maxDisplayY, // range of Y values
                        alienXPosition, alienLevelY, // start position
                        32, 32,  // width and height of frame
                        Content.Load<Texture2D>("Sprites/hitExplosion2727"), // Explosion texture
                        27, // width of each frame in the explosion animation
                        Content.Load<SoundEffect>("Sounds/AlienHit"), // explosion soundeffect
                        Rocket);
                    GameSprites.Add(alien);
                    AlienSprites.Add(alien);
                    alienXPosition = alienXPosition + alienWidth;
                }
                alienLevelY += levelGap;
            }

            Missile = new MissileSprite(
                    Content.Load<Texture2D>("Sprites/missile0320"),
                    0.005f,
                    60,    // 100 ticks to cros the screen
                    minDisplayX, maxDisplayX, // range of X values
                    minDisplayY, maxDisplayY, // range of Y values
                    alienXPosition, minDisplayY, // start position
                    3, 20,  // width and height of frame
                    Rocket,
                    AlienSprites);

            GameSprites.Add(Missile);

            musicSong = Content.Load<Song>("Sounds/StarlightMusic");
            gameOverSound = Content.Load<SoundEffect>("Sounds/StarlightGameOver");

            loadFont();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

#if FakeAccelerometer
            Accelerometer.Update();
#endif

            gamePad1 = GamePad.GetState(PlayerIndex.One);
            touchPad = TouchPanel.GetState();
            accellerometer = Accelerometer.GetState();

            switch (state)
            {
                case GameState.titleScreen:
                    Title.Update(this);
                    foreach (BaseSprite sprite in TitleSprites)
                    {
                        sprite.Update(this);
                    }
                    break;
                case GameState.playingGame:
                    foreach (BaseSprite sprite in GameSprites)
                    {
                        sprite.Update(this);
                    }
                    break;
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin();

            switch (state)
            {
                case GameState.titleScreen:
                    foreach (BaseSprite sprite in TitleSprites)
                    {
                        sprite.Draw(spriteBatch);
                    }
                    drawHighScore();
                    break;
                case GameState.playingGame:
                    foreach (BaseSprite sprite in GameSprites)
                    {
                        sprite.Draw(spriteBatch);
                    }
                    drawScore();
                    break;
            }

            spriteBatch.End();


            base.Draw(gameTime);
        }
    }
}
